/*
  This program adds Sex (boy, girl) to the program 04 Reliability 2 trials.sas;
  Simple test-retest reliability, allowing for simulation of different residuals 
in the two trials and individual responses in the second trial. Only one residual 
variance is estimated for the two trials, but a different residual variance is 
estimated for boys and girls.
  Modify the program as shown in the proc mixed step to play with the random effects.
  Values shown could be 100*log transformed &dep; hence SDs and mean effects are
approximate percents; e.g., 5 back-transforms to a factor of exp(5/100)=1.051 or 5.1%.
Back-transformation is not performed in this program.
*/
*b=boys, g=girls;
%let dep=VO2max; *the dependent variable;
%let meanb=380; *true grand mean in the first trial, back transforms to exp(380/100)=44.7;
%let meang=370; *true grand mean in the first trial, back transforms to exp(370/100)=40.4;
%let sdb=10; *true between-subject SD;
%let sdg=13; *true between-subject SD;
%let temb=3; *typical (standard) error of measurement;
%let temg=5; *typical (standard) error of measurement;
%let deltameanb=2; *change in mean Trial 2 minus Trial 1;
%let deltameang=3; *change in mean Trial 2 minus Trial 1;
%let extraerror1b=0; *more error on Trial 1, e.g., due to familiarization;
%let extraerror1g=0; *more error on Trial 1, e.g., due to familiarization;
%let extraerror2b=0; *more error on Trial 2, e.g., due to individual responses;
%let extraerror2g=0; *more error on Trial 2, e.g., due to individual responses;
%let ssizeb=20; *sample size;
%let ssizeg=20; *sample size;
%let decdep=1; *decimal places for simple stats of dependent variable;
%let deceff=1; *decimal places for SDs and effects;
%let tValue=3.5; *threshold for outliers with ~50 residuals. Use 4.0 for ~500, 4.5 for ~5000;
%let alpha=0.1; *0.1 gives 90% confidence limits;

*generate the data;
data dat1;
Sex="boy "; *note the space after the y;
do ChildID=1 to &ssizeb;
  True&dep=&meanb+&sdb*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&temb*rannor(0)+&extraerror1b*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameanb+&temb*rannor(0)+&extraerror2b*rannor(0);
  output;
  end;
Sex="girl";
do ChildID=&ssizeb+1 to &ssizeb+&ssizeg;
  True&dep=&meang+&sdg*rannor(0);
  Trial=1;
  Obsvd&dep=True&dep+&temg*rannor(0)+&extraerror1g*rannor(0);
  output;
  Trial=2;
  Obsvd&dep=True&dep+&deltameang+&temg*rannor(0)+&extraerror2g*rannor(0);
  output;
  end;

title "Raw data, first 10 observations (which will miss the girls)";
proc print data=dat1(obs=10);
format True&dep Obsvd&dep 5.&decdep;
run;

title "Simple stats";
proc means maxdec=&decdep;
var True&dep Obsvd&dep;
class Sex Trial;
run;


*generate means and SDs for plotting;
proc sort data=dat1;
by Sex Trial;

proc means noprint data=dat1;
var Obsvd&dep;
by Sex Trial;
output out=means mean=Mean std=SD;

data means1;
set means;
MeanPlus=Mean+SD;
MeanMinus=Mean-SD;

data datplot;
set dat1(in=a) means1;
*trick to get colored plots for individual values;
if a then do;
  if Sex="boy" then Obsvd&dep.boy=Obsvd&dep;
  else Obsvd&dep.girl=Obsvd&dep;
  end;
run;

ods graphics / reset width=10cm height=13cm imagemap attrpriority=none;
title "Line diagram of individual subjects";
title2 "and means and SDs for boys and girls";
proc sgplot data=datplot;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black)
  datalinepatterns=(solid dash dot);
*reg x=Age y=Obsvd&dep/nomarkers lineattrs=(thickness=1) group=Sex ;
scatter x=Trial y=Mean/filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Sex groupdisplay=cluster clusterwidth=0.1 name="points"; *duplication of points to get legend;
series x=Trial y=Obsvd&dep.boy/lineattrs=(pattern=solid color=lightskyblue) group=ChildID;
series x=Trial y=Obsvd&dep.girl/lineattrs=(pattern=solid color=pink) group=ChildID;
series x=Trial y=Mean/lineattrs=(pattern=solid color=black)
  group=Sex groupdisplay=cluster clusterwidth=0.1;
scatter x=Trial y=Mean/errorbarattrs=(color=black) yerrorlower=MeanMinus yerrorupper=MeanPlus
  filledoutlinedmarkers markeroutlineattrs=(color=black) markerattrs=(size=12) 
  group=Sex groupdisplay=cluster clusterwidth=0.1;
xaxis values=(1,2) offsetmin=0.1 offsetmax=0.1;
yaxis label="&dep" labelpos=top;
legenditem type=line name="linec"/lineattrs=(color=lightskyblue) label="boy"; 
legenditem type=line name="linee"/lineattrs=(color=pink) label="girl";
keylegend "points" "linec" "linee";
run;

*Set up dummy variables for estimating extra variance in Trial 1 or Trial 2;
data dat2;
set dat1;
XvarTrial1=0;
if Trial=1 then XvarTrial1=1;
XvarTrial2=0;
if Trial=2 then XvarTrial2=1;

*clear datasets from any previous proc mixed, to avoid confusion if next proc mixed fails;
data clev cov est pred solf solr lsm lsmdiff;

ods select none;
title "Reliability mixed model plus sex. Boys, Girls: Sample size=&ssizeb, &ssizeg; True mean=&meanb, &meang; true between SD=&sdb, &sdg";
title2 "Delta mean Trial2-1=&deltameanb, &deltameang; Typical error=&temb, &temg; Extra error Trial1=&extraerror1b, &extraerror1g, Trial2=&extraerror2b, &extraerror2g";
proc mixed data=dat2 covtest cl alpha=&alpha nobound;
class ChildID Sex Trial; *always make the order here the same in the model statement;
model Obsvd&dep=Sex*Trial/noint ddfm=sat solution cl alpha=&alpha residual outp=pred alphap=&alpha;
random ChildID/group=Sex s cl alpha=&alpha;
repeated/group=Sex;
*repeated/group=Trial;
*repeated/group=Sex*Trial;
*repeated Trial/subject=ChildID type=un group=Sex;
estimate "Trial 2-1 boys" Sex*Trial -1 1  0 0/cl alpha=&alpha;
estimate "Trial 2-1 girls" Sex*Trial 0 0  -1 1/cl alpha=&alpha;
estimate "Trial 2-1 boys-girls" Sex*Trial -1 1  1 -1/cl alpha=&alpha;
estimate "Trial 2-1 mean" Sex*Trial -1 1  -1 1/cl alpha=&alpha divisor=2;
lsmeans Sex*Trial/diff cl alpha=&alpha;
ods output classlevels=clev;
ods output covparms=cov;
ods output solutionf=solf;
ods output solutionr=solr;
ods output estimates=est;
ods output lsmeans=lsm;
ods output diffs=lsmdiff;
run;
ods select all;

title3 "Class levels";
proc print data=clev;
run;

title3 "Covparms (random effects)";
title4 "Check for proper estimation, especially the Residual";
proc print data=cov;
run;

data cov1;
length Group $ 10; **in case there is a group= in the random effects;
set cov;
array a Estimate Lower Upper; 
do over a;
  a=sign(a)*sqrt(sign(a)*a); *to allow for negative variance, with the random statement and nobound;
  end;
if Covparm="Residual" then CLtd=sqrt(Upper/Lower); *Confidence Limits in times/divide format. This is for residuals and it is a good approximation;
else CLpmApprox=(Upper-Lower)/2; *Confidence Limits in +/- format. This is for the other random effects with random and nobound;
DegFree=2*ZValue**2; *Degrees of freedom;
run;

title3 "Covparms (random effects) as SD";
proc print data=cov1;
var  CovParm Group Estimate CLtd CLpmApprox Lower Upper Alpha DegFree;
format Estimate Lower Upper CLpmApprox 5.&deceff CLtd 5.2 DegFree 5.1; 
run;

title3 "Fixed-effects solution";
proc print data=solf;
run;

data est1;
set est;
CLpm=(Upper-Lower)/2;
if Estimate=0 then Estimate=.;

title3 "Fixed-effect estimates";
proc print data=est1;
var Label Estimate CLpm Lower Upper Alpha DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsm1;
set lsm;
CLpm=(Upper-Lower)/2;

title3 "Least-squares means";
proc print data=lsm1;
var Effect Sex Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

data lsmdiff1;
set lsmdiff;
CLpm=(Upper-Lower)/2;

title3 "Differences in least-squares means";
proc print data=lsmdiff1;
var Effect Sex _Sex Trial _Trial Estimate CLpm Lower Upper DF;
format Estimate CLpm Lower Upper 5.&deceff DF 5.0;
run;

title3 "Residuals vs predicteds";
ods graphics / reset width=12cm height=10cm imagemap attrpriority=none;
proc sgplot data=pred; *uniform=all;
styleattrs
  datacolors=(lightskyblue pink red orange yellow lightgreen lightskyblue magenta lightgrey)
  datasymbols=(circlefilled trianglefilled triangledownfilled squarefilled diamondfilled starfilled HomeDownFilled)
  datacontrastcolors=(black);
refline 0/axis=y lineattrs=(pattern=dot thickness=1 color=black);
scatter x=pred y=Resid/filledoutlinedmarkers markerattrs=(size=10) group=Sex;
run;
ods graphics / reset;

title2 "Outliers (Standardized residual >&tvalue)";
title3 "Pred is the value of the dependent predicted by the model";
data pred1;
set pred;
if abs(StudentResid)>&tvalue;

proc print data=pred1;
format Obsvd&dep Pred Resid 5.&decdep StudentResid 5.1;
run;

title3 "Random-effects solution, first 10 observations";
proc print data=solr(obs=10);
format Estimate StdErrPred Lower Upper 5.&deceff;
run;









